package it.inaf.oats.ia2.tap.tsm.model;

import it.inaf.oats.ia2.tap.tsm.datalayer.TableDL;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author pietro
 */
public class Table extends EditablesContainer implements IDbEntity, IDbEntitiesContainer, Serializable {

    private Status status;

    private final Schema schema;
    private final String tableName;
    private final Map<String, Column> columnsMap;

    public String table_type;
    private String utype;
    private String description;

    private String selectedColumn;

    public Table(Connection connection, String tapSchemaName, Schema schema, String tableName, boolean toCreate) throws SQLException {
        super(UTYPE, DESCRIPTION);
        
        this.tableName = tableName;
        this.schema = schema;
        this.columnsMap = new HashMap<String, Column>();
        status = Status.NOT_SELECTED;

        TableDL.loadAllColumns(connection, tapSchemaName, schema.getName(), (this));

        if (!toCreate) {
            TableDL.loadSavedColumns(connection, tapSchemaName, schema.getName(), (this));
        }
    }

    public Map<String, Column> getColumnsMap() {
        return columnsMap;
    }

    @Override
    public void addEntity(String key, IDbEntity column) {
        columnsMap.put(key, (Column) column);
    }

    @Override
    public ArrayList<String> getAddables() {
        ArrayList<String> addablesColumns = new ArrayList<String>();

        for (Column column : columnsMap.values()) {
            if (column.getStatus() == Status.NOT_SELECTED) {
                addablesColumns.add(column.getName());
            }
        }

        return addablesColumns;
    }

    @Override
    public ArrayList<IDbEntity> getAllEntities() {
        ArrayList<IDbEntity> list = new ArrayList<IDbEntity>();
        for (Column column : columnsMap.values()) {
            if (column.getStatus() != Status.NOT_SELECTED) {
                list.add(column);
            }
        }
        return list;
    }

    public Column getColumn(String columnName) {
        return columnsMap.get(columnName);
    }

    @Override
    public String getName() {
        return tableName;
    }

    public String getFullName() {
        return schema.getName() + "." + tableName;
    }

    public String getUtype() {
        return utype;
    }

    public void setUtype(String utype) {
        this.utype = utype;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    @Override
    public Status getStatus() {
        return status;
    }

    @Override
    public void setStatus(Status status) {
        this.status = status;
    }

    @Override
    public void loadEntities(Connection connection, ArrayList<String> selectedEntities) throws SQLException {
        for (String columnName : selectedEntities) {
            getColumn(columnName).setStatus(Status.SELECTED_TO_SAVE);
        }
    }

    @Override
    public void removeEntity(String name) {
        Column column = getColumn(name);
        if (column.getStatus() == Status.SELECTED_SAVED) {
            column.setStatus(Status.TO_REMOVE);
        } else {
            column.setStatus(Status.NOT_SELECTED);
        }
    }

    public boolean isForeignKey(String columnName) {
        return schema.isForeignKey(getFullName(), columnName);
    }

    @Override
    public IDbEntity getSelected() {
        return columnsMap.get(selectedColumn);
    }

    @Override
    public void select(String name) {
        selectedColumn = name;
    }

    @Override
    public String getOriginalValue(String key) {
        if (key.equals(UTYPE)) {
            return this.getUtype();
        }
        if (key.equals(DESCRIPTION)) {
            return this.getDescription();
        }
        return null;
    }
}
