package it.inaf.oats.ia2.tap.tsm.rest;

import com.google.gson.GsonBuilder;
import it.inaf.oats.ia2.tap.tsm.model.Credentials;
import it.inaf.oats.ia2.tap.tsm.model.EditablesContainer;
import it.inaf.oats.ia2.tap.tsm.model.IDbEntitiesContainer;
import it.inaf.oats.ia2.tap.tsm.model.IDbEntity;
import it.inaf.oats.ia2.tap.tsm.model.Schema;
import it.inaf.oats.ia2.tap.tsm.model.Table;
import it.inaf.oats.ia2.tap.tsm.model.TapSchema;
import it.inaf.oats.ia2.tap.tsm.pagemodel.Tab;
import javax.ws.rs.core.Context;
import javax.ws.rs.Consumes;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;

/**
 * REST Web Service
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@Path("tapschema")
public class TapSchemaResource {

    // containers names
    private static final String TAP_SCHEMA = "tapSchema";
    private static final String SCHEMA = "schema";
    private static final String TABLE = "table";

    // models names
    private static final String SCHEMAS = "schemas";
    private static final String SCHEMA_VALUES = "schemaValues";
    private static final String TABLES = "tables";
    private static final String TABLE_VALUES = "tableValues";
    private static final String COLUMNS = "columns";
    private static final String COLUMN_VALUES = "columnValues";
    private static final String SELECTOR_MODAL = "selectorModal";

    class EditableField {

        String value;
        boolean isChanged;

        EditableField(String key, EditablesContainer container) {
            this.value = container.getValue(key);
            this.isChanged = container.isChanged(key);
        }
    }

    @Context
    private HttpServletRequest request;

    private TapSchema getTapSchema() {
        TapSchema tapSchema = null;
        HttpSession session = request.getSession();
        if (session.getAttribute(TAP_SCHEMA) == null) {
            Credentials credentials = new Credentials("localhost", 3306, "root", "root");
            Connection connection = null;
            try {
                connection = credentials.getConnection();

                tapSchema = new TapSchema(connection, "vlkb_voinfo_schema");
                session.setAttribute(TAP_SCHEMA, tapSchema);
            } catch (SQLException e) {
                e.printStackTrace(System.err);
            } finally {
                try {
                    if (connection != null) {
                        connection.close();
                    }
                } catch (SQLException e) {
                    e.printStackTrace(System.err);
                }
            }
        } else {
            tapSchema = (TapSchema) session.getAttribute(TAP_SCHEMA);
        }
        return tapSchema;
    }

    /**
     * Creates a new instance of TapSchemaResource
     */
    public TapSchemaResource() {
    }

    @POST
    @Path("models")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public String getPartialModels(List<String> list) {
        return getModels(list);
    }

    private String getModels(String... params) {
        return getModels(Arrays.asList(params));
    }

    private String getModels(List<String> list) {
        HashMap<String, Object> models = new HashMap<String, Object>();

        TapSchema tapSchema = getTapSchema();

        if (modelSelected(SCHEMAS, list)) {
            models.put(SCHEMAS, getAllEntities(tapSchema));
        }

        if (modelSelected(TABLES, list)) {
            models.put(TABLES, getAllEntities(tapSchema.getSelectedSchema()));
        }

        if (modelSelected(COLUMNS, list)) {
            models.put(COLUMNS, getAllEntities(tapSchema.getSelectedTable()));
        }

        if (modelSelected(SCHEMA_VALUES, list)) {
            models.put(SCHEMA_VALUES, getEditablesFields(tapSchema.getSelectedSchema()));
        }
        if (modelSelected(TABLE_VALUES, list)) {
            models.put(TABLE_VALUES, getEditablesFields(tapSchema.getSelectedTable()));
        }
        if (modelSelected(COLUMN_VALUES, list)) {
            models.put(COLUMN_VALUES, getEditablesFields(tapSchema.getSelectedColumn()));
        }

        return new GsonBuilder().create().toJson(models);
    }

    private ArrayList<Tab> getAllEntities(IDbEntitiesContainer container) {
        if (container == null) {
            return null;
        }

        ArrayList<Tab> entities = new ArrayList<Tab>();
        String selectedEntity = container.getSelected() == null ? null : container.getSelected().getName();
        for (IDbEntity entity : container.getAllEntities()) {
            entities.add(new Tab(entity, entity.getName().equals(selectedEntity)));
        }
        return entities;
    }

    private HashMap<String, EditableField> getEditablesFields(EditablesContainer container) {
        if (container == null) {
            return null;
        }

        HashMap<String, EditableField> fields = new HashMap<String, EditableField>();
        for (String key : container.getSupportedValues()) {
            fields.put(key, new EditableField(key, container));
        }
        return fields;
    }

    @POST
    @Path("select")
    public String select(@FormParam("containerName") String containerName, @FormParam("entityName") String entityName) {
        IDbEntitiesContainer container = getEntitiesContainer(containerName);
        container.select(entityName);

        if (containerName.equals(TAP_SCHEMA)) {
            return getModels(SCHEMA_VALUES, TABLES, TABLE_VALUES, COLUMNS, COLUMN_VALUES);
        }
        if (containerName.equals(SCHEMA)) {
            return getModels(TABLE_VALUES, COLUMNS, COLUMN_VALUES);
        }
        if (containerName.equals(TABLE)) {
            return getModels(COLUMN_VALUES);
        }

        return null;
    }

    private boolean modelSelected(String modelName, List<String> list) {
        for (String model : list) {
            if (model.equals(modelName)) {
                return true;
            }
        }
        return false;
    }

    private IDbEntitiesContainer getEntitiesContainer(String name) {
        TapSchema tapSchema = getTapSchema();
        if (name.equals(TAP_SCHEMA)) {
            return tapSchema;
        }
        if (name.equals(SCHEMA)) {
            return tapSchema.getSelectedSchema();
        }
        if (name.equals(TABLE)) {
            return tapSchema.getSelectedTable();
        }
        return null;
    }
}
